# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: token.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0btoken.proto\"J\n\x05Token\x12\x0f\n\x07user_id\x18\x01 \x01(\r\x12\x12\n\nproject_id\x18\x02 \x01(\r\x12\x0f\n\x07methods\x18\x03 \x01(\t\x12\x0b\n\x03\x65xp\x18\x04 \x01(\x01\x62\x06proto3')



_TOKEN = DESCRIPTOR.message_types_by_name['Token']
Token = _reflection.GeneratedProtocolMessageType('Token', (_message.Message,), {
  'DESCRIPTOR' : _TOKEN,
  '__module__' : 'token_pb2'
  # @@protoc_insertion_point(class_scope:Token)
  })
_sym_db.RegisterMessage(Token)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TOKEN._serialized_start=15
  _TOKEN._serialized_end=89
# @@protoc_insertion_point(module_scope)



import asyncio
import concurrent.futures
import functools
import inspect
import os
import pathlib
import tempfile
import traceback
import typing
import uuid

import fastapi
from starlette import status
from mmf_meta.descriptors import DescriptorBase
from mmf_meta.core import Target
from .fastapi_tasks import make_tasks
from .patch_upload_file import patch_upload_file

patch_upload_file()

tasks_results = pathlib.Path.cwd() / "data_results"
os.makedirs(tasks_results, exist_ok=True)


def _run_foo(t: Target, sig: inspect.Signature, args: dict, result_file: str = None):
    foo = t.foo
    for n in args:
        desc: DescriptorBase = sig.parameters[n].default
        if desc._is_file:
            args[n] = desc.load_file(args[n])
    ret = foo(**args)
    if t.returns is not None and t.returns._is_file and ret is not None:
        t.returns.to_file(ret, target_file=result_file)
        return result_file
    else:
        return ret


def _get_fastapi_spec(d: DescriptorBase):
    a, d = d.fastapi_descriptor
    return {"annotation": a, "default": d}


def wrap_fastapi(
    t: Target, pool, app: fastapi.FastAPI, task_results: dict, cancel_tasks: dict
):
    """
    Оборачиваем таргет как fastapi-совместимую функцию

    :param t:
    :param pool:
    :param app:
    :param task_results: Mapping для хранения результатов исполнения, результат - Future в текущем исполнении
    :param cancel_tasks: Список Future для отмены исполнения задачи.
    :return:
    """
    orig_sig = inspect.signature(t.foo)
    sig = orig_sig.replace(
        parameters=[
            p.replace(**_get_fastapi_spec(p.default))
            for p in orig_sig.parameters.values()
        ]
        + [
            inspect.Parameter(
                name="_back",
                kind=inspect.Parameter.KEYWORD_ONLY,
                annotation=fastapi.BackgroundTasks,
            ),
            inspect.Parameter(
                name="as_task",
                kind=inspect.Parameter.KEYWORD_ONLY,
                annotation=typing.Optional[bool],
                default=fastapi.Query(
                    False,
                    description="Если true, то задача будет поставлена в очередь, в таком случае в обязательном порядке "
                    "нужно указать заголовок TASK-ID. Запросить статус по выполнению задачи можно с помощью GET /task "
                    "Если TASK-ID пересекается с одной из задач в очереди, эта задача"
                    " отменяется, вместо нее встает новая",
                ),
            ),
            inspect.Parameter(
                name="task_id",
                kind=inspect.Parameter.KEYWORD_ONLY,
                annotation=typing.Optional[str],
                default=fastapi.Header(None, description="ID задачи для as_task=True"),
            ),
        ]
    )

    async def run_task(foo, task_id: str):
        loop = asyncio.get_event_loop()

        task: asyncio.Future = loop.run_in_executor(pool, foo)
        cancel: asyncio.Future = cancel_tasks.get(task_id)
        res: asyncio.Future = task_results.get(task_id, None)

        def on_done(_res: asyncio.Future):
            if res.done():
                return
            try:
                ret = _res.result()
                if (
                    t.returns is not None
                    and t.returns._is_file
                    and isinstance(ret, os.PathLike)
                ):
                    ret = fastapi.responses.FileResponse(
                        path=ret,
                    )
                else:
                    ret = {"result": ret}
                    ret = fastapi.responses.ORJSONResponse(content=ret)
                res.set_result(ret)
            except Exception as exc:
                res.set_exception(exc)

        task.add_done_callback(on_done)
        try:
            await asyncio.wait([task, cancel], return_when=asyncio.FIRST_COMPLETED)
            if cancel.done():
                task.cancel()
                if not task.done():
                    res.set_exception(asyncio.CancelledError())
        finally:
            cancel_tasks.pop(task_id, None)

    async def wrapper(**data):
        back: fastapi.BackgroundTasks = data.pop("_back", None)
        as_task: bool = data.pop("as_task", False)
        task_id: str = data.pop("task_id", None)
        for n, v in data.items():
            desc: DescriptorBase = orig_sig.parameters[n].default
            if desc._is_file:
                data[n] = v.get_file_name()
        if as_task and not task_id:
            raise fastapi.HTTPException(
                status_code=status.HTTP_422_UNPROCESSABLE_ENTITY,
                detail="must provide TASK-ID header if use as_task=True",
            )
        loop = asyncio.get_event_loop()
        is_file = t.returns and t.returns._is_file

        if is_file:
            result_file = (
                tasks_results
                / f"task.{task_id if task_id else uuid.uuid4().hex}.{t.returns.out_format}"
            )
        else:
            result_file = None
        foo = functools.partial(
            _run_foo,
            t,
            orig_sig,
            data,
            result_file,
        )
        if not as_task:
            try:
                ret = await loop.run_in_executor(pool, foo)
            except Exception as exc:
                raise
                raise fastapi.HTTPException(
                    status_code=status.HTTP_422_UNPROCESSABLE_ENTITY,
                    detail={
                        "err": True,
                        "msg": str(exc),
                        "trace": traceback.format_exc(),
                    },
                )
            if t.returns and t.returns._is_file and isinstance(ret, os.PathLike):
                return fastapi.responses.FileResponse(ret)
            else:
                return fastapi.responses.ORJSONResponse({"result": ret})
        else:
            cancel: asyncio.Future = cancel_tasks.pop(task_id, None)
            result: asyncio.Future = task_results.pop(task_id, None)
            if cancel:
                cancel.set_result(True)
            if result:
                result.set_exception(asyncio.CancelledError())

            cancel_tasks[task_id] = loop.create_future()
            task_results[task_id] = loop.create_future()
            back.add_task(functools.partial(run_task, foo, task_id))
            return fastapi.Response(status_code=200)
        # TODO: тут еще нужно поработать над get_files

    wrapper.__signature__ = sig
    wrapper.__name__ = t.foo.__name__
    wrapper = app.post(
        f"/{t.foo.__name__}",
        description=t.description,
        response_description=t.returns.description if t.returns else None,
        name=t.name,
        tags=["targets"],
    )(wrapper)
    return wrapper


def get_fastapi_app(targets: typing.List[Target], n_proc=1):
    app = fastapi.FastAPI()
    pool = concurrent.futures.ProcessPoolExecutor(n_proc)
    task_results = {}
    cancel_tasks = {}
    for t in targets:
        wrap_fastapi(t, pool, app, task_results=task_results, cancel_tasks=cancel_tasks)

    @app.get("/health", tags=["sys"])
    async def health():
        return fastapi.Response(status_code=200)

    make_tasks(app, task_results=task_results, cancel_tasks=cancel_tasks)
    return app



from pydantic import BaseSettings, BaseModel


class RabbitConfig(BaseModel):
    user: str = "core"
    password: str = None
    host: str = "localhost"

    @property
    def con_string(self):
        return f"amqp://{self.user}:{self.password}@{self.host}/"


class S3Settings(BaseModel):
    aws_access_key_id: str = "minioadmin"
    aws_secret_access_key: str = "minioadmin"
    endpoint_url: str = "http://localhost:9000"


class Config(BaseSettings):
    rabbit: RabbitConfig = None
    s3: S3Settings = None
    exchange_name: str = "tasks"
    n_workers: int = 1
    coordinator_path: str = "/run/coordinator"
    logger_path: str = "/run/logger"
    consumer_user: str = "appuser"
    interpreter: str = "/home/appuser/env/bin/python3"
    cwd: str = "/home/appuser/app"
    project_meta: str = "mmf.yml"
    consumer_script: str = "-m ipc consume"
    secret_key: str = ""
    public_key: str = "key.pub"
    project_id: int = 0

    class Config:
        env_file_encoding = "utf-8"
        extra = "allow"
        env_nested_delimiter = "__"


config = Config(_env_file=".secrets.env")



import tempfile
from aiofile import async_open
from starlette.datastructures import UploadFile

orig = UploadFile.__init__


def custom_init(self: UploadFile, *args, **kwargs):
    orig(self, *args, **kwargs)
    filename = self.filename or ""
    *_, ext = filename.split(".")
    self.file = async_open(tempfile.NamedTemporaryFile(suffix=f".{ext}"))


async def custom_write(self: UploadFile, data):
    await self.file.write(data)


async def custom_read(self: UploadFile, size: int = -1):
    return self.file.read(size)


async def custom_seek(self, offset: int):
    try:
        return self.file.seak(offset)
    except Exception as exc:
        pass


async def custom_close(self):
    await self.file.close()


def get_file_name(self: UploadFile):
    self.filename
    return self.file.file.name


def patch_upload_file():
    UploadFile.__init__ = custom_init
    UploadFile.write = custom_write
    UploadFile.read = custom_read
    UploadFile.seek = custom_seek
    UploadFile.close = custom_close
    UploadFile.get_file_name = get_file_name



import asyncio
import traceback
import typing
import fastapi.routing
from starlette import status


def make_tasks(app: fastapi.FastAPI, task_results: dict, cancel_tasks: dict):
    @app.get("/task", tags=["tasks"])
    async def get_task(
        task_id: str = fastapi.Header(..., description="ID задачи"),
        auto_remove: bool = fastapi.Query(
            True, description="автоматически удалить задачу"
        ),
    ):
        """
        Возвращает результат выполнения задачи по завершению выполнения

        :param task_id:
        :return:
        """
        ret = task_results.get(task_id)
        try:
            ret = await ret
            if auto_remove:
                task_results.pop(task_id, None)
                cancel_tasks.pop(task_id, None)
            return ret
        except Exception as exc:
            raise fastapi.HTTPException(
                status_code=status.HTTP_422_UNPROCESSABLE_ENTITY,
                detail={"err": True, "msg": str(exc), "trace": traceback.format_exc()},
            )

    @app.delete("/task", tags=["tasks"])
    async def delete_task(task_id: str = fastapi.Header(..., description="ID задачи")):
        """
        Удаляет/отменяет задачу

        :param task_id:
        :return:
        """
        ret: asyncio.Future = task_results.pop(task_id, None)
        if ret is None:
            return
        cancel: asyncio.Future = cancel_tasks.get(task_id, None)
        if ret.done():
            return
        elif cancel:
            cancel.set_result(True)
            ret.set_exception(asyncio.CancelledError())
            return

    @app.get("/tasks", tags=["tasks"])
    async def get_tasks(
        task_id: typing.Optional[str] = fastapi.Header(
            None,
            description="ID задачи, в данном случае не обязательный "
            "параметр, используется только, если нужно "
            "привязать запрос к определенной реплике",
        )
    ):
        """
        Список всех задач с их статусами
        :return:
        """
        done = []
        pending = []
        for task_id, task in task_results.items():
            if task.done():
                done.append(task_id)
            else:
                pending.append(task_id)
        return fastapi.responses.ORJSONResponse({"done": done, "pending": pending})

    @app.delete("/tasks", tags=["tasks"])
    async def get_tasks(
        task_id: typing.Optional[str] = fastapi.Header(
            None,
            description="ID задачи, в данном случае не обязательный "
            "параметр, используется только, если нужно "
            "привязать запрос к определенной реплике",
        ),
        only_done: bool = fastapi.Query(
            True, description="Удаляются только завершенные задачи"
        ),
    ):
        """
        Удаляются все задачи
        :return:
        """
        done = []
        pending = []
        for task_id, task in task_results.items():
            if task.done():
                done.append(task_id)
            else:
                pending.append(task_id)
        for task_id in done:
            task_results.pop(task_id, None)
            cancel_tasks.pop(task_id, None)
        if not only_done:
            for task_id in pending:
                task: asyncio.Future = task_results.pop(task_id, None)
                task.set_exception(asyncio.CancelledError())
                cancel_tasks.pop(task_id, None)
        return fastapi.responses.Response(status_code=200)



from libc.stdint cimport *
from libc.string cimport *
from cpython.ref cimport PyObject

from pyrobuf_list cimport *
from pyrobuf_util cimport *

import json

cdef class Token:


    cdef uint32_t _user_id
    cpdef _user_id__reset(self)
    cdef uint32_t _project_id
    cpdef _project_id__reset(self)
    cdef str _methods
    cpdef _methods__reset(self)
    cdef double _exp
    cpdef _exp__reset(self)


    cdef uint64_t __field_bitmap0

    cdef public bint _is_present_in_parent
    cdef bytes _cached_serialization

    cdef object _listener

    cpdef void reset(self)

    cpdef void Clear(self)
    cpdef void ClearField(self, field_name)
    cpdef void CopyFrom(self, Token other_msg)
    cpdef bint HasField(self, field_name) except -1
    cpdef bint IsInitialized(self)
    cpdef void MergeFrom(self, Token other_msg)
    cpdef int MergeFromString(self, data, size=*) except -1
    cpdef int ParseFromString(self, data, size=*, bint reset=*, bint cache=*) except -1
    cpdef bytes SerializeToString(self, bint cache=*)
    cpdef bytes SerializePartialToString(self)

    cdef void _clearfield(self, field_name)
    cdef int _protobuf_deserialize(self, const unsigned char *memory, int size, bint cache)

    cdef void _protobuf_serialize(self, bytearray buf, bint cache)

    cpdef void _Modified(self)


import base64
import functools
import operator
import time
import typing
import fastapi
import orjson
from cryptography.exceptions import InvalidSignature
from cryptography.hazmat.primitives import serialization
from starlette import status
from dataclasses import dataclass
from token_proto import Token as _Token
from .config import config

with open(config.public_key, "br") as f:
    public_key = serialization.load_pem_public_key(f.read())

_token = _Token()


@dataclass
class Token:
    user_id: int
    project_id: int
    methods: typing.Union[typing.List[str], str]
    exp: float

    @classmethod
    def from_proto(cls, data: bytes):
        token = _Token.FromString(data)
        # ret = token.SerializeToDict()
        return cls(
            user_id=token.user_id,
            project_id=token.project_id,
            methods=token.methods,
            exp=token.exp,
        )

    def to_proto(self, encode=False):
        ret = _Token()
        ret.user_id = self.user_id
        ret.project_id = self.project_id
        ret.methods = self.methods
        ret.exp = self.exp
        if encode:
            return base64.b64encode(ret.SerializeToString())
        else:
            return ret.SerializeToString()

    def to_json(self):
        ret = orjson.dumps(self)
        return ret

    def to_m(self):
        m = self.methods.encode()
        ret = (
            self.user_id.to_bytes(4, "big", signed=False),
            self.project_id.to_bytes(4, "big", signed=False),
            int(self.exp).to_bytes(4, "big", signed=False),
            len(m).to_bytes(1, "big", signed=False),
            m,
        )
        return functools.reduce(operator.add, ret)

    @classmethod
    def from_json(cls, v):
        ret = orjson.loads(v)
        return cls(
            **{
                "user_id": ret["user_id"],
                "project_id": ret["project_id"],
                "methods": ret["methods"],
                "exp": ret["exp"],
            }
        )


def verify_b64_token(data: str, method: str):
    return verify_token(base64.b64decode(data), method)


def verify_token(data: bytes, method: str):
    """
    Проверяет подпись, сверяет id проекта, запрошенный метод и срок валидности
    :param data:
    :param method:
    :return:
    """
    sig = data[:256]
    token = data[256:]
    ex = fastapi.HTTPException(status_code=status.HTTP_401_UNAUTHORIZED)
    try:
        public_key.verify(
            sig,
            token,
        )
        token = Token.from_proto(token)
        if (
            token.project_id != config.project_id
            or method not in token.methods
            or token.exp < time.time()
        ):
            raise ex
        return token
    except InvalidSignature:
        raise ex



import asyncio
import importlib
import os
import sys
import click
from mmf_meta.core import scan
from .config import config
from .rabbit_wrapper import serve_rabbitmq


@click.group()
def cli():
    sys.path.append(os.getcwd())
    return


@cli.command(name="serve-rabbit")
@click.argument("module")
@click.option("--n_proc", default=1)
@click.option("--queue_name")
@click.option("--results_exchange")
def serve_rabbit(module: str, queue_name: str, results_exchange: str, n_proc):
    """
    Прослушивание задач из очереди
    """
    module = importlib.import_module(module.replace(".py", ""))
    targets, _ = scan()
    loop = asyncio.get_event_loop()
    loop.run_until_complete(
        serve_rabbitmq(
            n_proc=n_proc,
            queue_name=queue_name,
            results_exchange=results_exchange,
            rabbit_params=config.rabbit.dict(),
            targets=targets,
        )
    )



import asyncio
import concurrent.futures
import inspect
import logging
import traceback
import typing
from http.client import HTTPResponse
from urllib import request

import orjson
import requests
from aio_pika import connect_robust, RobustChannel, Exchange, IncomingMessage, Message

from mmf_meta.core import Target
from mmf_meta.descriptors import DescriptorBase, JsonFile, Dict

lg = logging.getLogger()


def _get_files(data, sig: inspect.Signature):
    args = {}
    for n, v in data.items():
        desc = sig.parameters.get(n)
        if desc is None:
            continue
        elif isinstance(desc.default, DescriptorBase) and desc.default.is_file:
            args[n] = desc.default.load_url(data[n])
        else:
            args[n] = v
    return args


def wrap_rabbit_s3(t: Target, msg: bytes, content_type: str):
    """
    Оборачиваем таргет как функцию, готовую принимать сообщения от rabbitmq
    Все файлы в таком случае будут ожидаться как ссылки на сетевое хранилище.
    Для s3 необходимо передавать presigned url
    Если функция возвращает файл, необходимо так же передать поле _ret_url, содержащее
    presigned url для загрузки итоговых данных.

    :param t:
    :return:
    """
    sig = inspect.signature(t.foo)

    if content_type == "json":
        data = orjson.loads(msg)
    else:
        raise TypeError(f"not compatible content_type {content_type}")
    if t.returns and t.returns.is_file and "_ret_url" not in data:
        raise ValueError(
            f"{t.name} returns file, ret_url must be provided in order to upload results"
        )
    args = _get_files(data, sig)
    ret = t.foo(**args)
    if t.returns:
        if t.returns.is_file:
            if isinstance(t.returns, JsonFile):
                if not t.returns.to_s3:
                    return orjson.dumps(ret)
            url: dict = data["_ret_url"]
            key: str = url["fields"]["key"]
            *_, ext = key.lower().split(".")
            ret = t.returns.to_file(ret, ext=ext)
            ret.seek(0)
            files = {"file": (url["fields"]["key"], ret)}
            resp = requests.post(url["url"], data=url["fields"], files=files)
            lg.debug("sending result to %s", url)

            if resp.status_code != 204:
                raise RuntimeError(
                    f"could not upload result to {url}, status: {resp.status_code}, info: {resp.content}"
                )
            return b"ok"
        elif isinstance(t.returns, Dict) and content_type == "json":
            if not isinstance(ret, dict):
                raise TypeError(
                    f"return value is expected to be dict-like, but is {type(ret)} instead"
                )
            return orjson.dumps(ret)
        else:
            return b"ok"

    return


async def serve_rabbitmq(
    n_proc: int,
    queue_name: str,
    targets: typing.List[Target],
    rabbit_params: dict,
    results_exchange: str,
):
    user = rabbit_params["user"]
    password = rabbit_params["password"]
    host = rabbit_params["host"]
    lg.info("connecting %s@%s", user, host)
    connection = await connect_robust(f"amqp://{user}:{password}@{host}/")
    channel: RobustChannel = await connection.channel()
    exchange: Exchange = await channel.get_exchange(results_exchange)
    lg.info("connecting %s", results_exchange)
    await channel.set_qos(prefetch_count=1)
    queue = await channel.get_queue(queue_name)
    lg.info("connecting %s", queue_name)
    targets = {t.name: t for t in targets}
    loop = asyncio.get_event_loop()
    with concurrent.futures.ThreadPoolExecutor(n_proc) as pool:
        async with queue.iterator() as queue_iter:
            async for message in queue_iter:
                async with message.process():
                    message: IncomingMessage
                    lg.debug(f"process message %s", message)
                    task_id: str = message.headers.get("task-id", None)
                    if task_id is None:
                        continue
                    try:
                        target = targets.get(message.headers.get("target"))
                        if target is None:
                            raise KeyError(f"target with key {target} dows not exists")
                        lg.debug("run %s", target)
                        ret = await loop.run_in_executor(
                            pool,
                            wrap_rabbit_s3,
                            target,
                            message.body,
                            message.content_type,
                        )
                    except Exception as exc:
                        lg.exception("while processing %s", message)
                        if message.content_type == "json":
                            ret = orjson.dumps(
                                {
                                    "error": True,
                                    "trace": traceback.format_exc(),
                                    "msg": str(exc),
                                }
                            )
                        else:
                            ret = f"ERROR: {exc}".encode()
                    lg.debug("sending results %s", ret)
                    await exchange.publish(
                        Message(ret, headers={"task-id": task_id}), routing_key=""
                    )



from libc.stdint cimport *
from libc.string cimport *

from pyrobuf_list cimport *
from pyrobuf_util cimport *

import base64
import json
import warnings

class DecodeError(Exception):
    pass

cdef class Token:

    def __cinit__(self):
        self._listener = noop_listener

    

    def __init__(self, **kwargs):
        self.reset()
        if kwargs:
            for field_name, field_value in kwargs.items():
                try:
                    setattr(self, field_name, field_value)
                except AttributeError:
                    raise ValueError('Protocol message has no "%s" field.' % (field_name,))
        return

    def __str__(self):
        fields = [
                          'user_id',
                          'project_id',
                          'methods',
                          'exp',]
        components = ['{0}: {1}'.format(field, getattr(self, field)) for field in fields]
        messages = []
        for message in messages:
            components.append('{0}: {{'.format(message))
            for line in str(getattr(self, message)).split('\n'):
                components.append('  {0}'.format(line))
            components.append('}')
        return '\n'.join(components)

    
    cpdef _user_id__reset(self):
        self._user_id = 0
        self.__field_bitmap0 &= ~1
    cpdef _project_id__reset(self):
        self._project_id = 0
        self.__field_bitmap0 &= ~2
    cpdef _methods__reset(self):
        self._methods = ""
        self.__field_bitmap0 &= ~4
    cpdef _exp__reset(self):
        self._exp = 0
        self.__field_bitmap0 &= ~8

    cpdef void reset(self):
        # reset values and populate defaults
    
        self._user_id__reset()
        self._project_id__reset()
        self._methods__reset()
        self._exp__reset()
        return

    
    @property
    def user_id(self):
        return self._user_id

    @user_id.setter
    def user_id(self, value):
        self.__field_bitmap0 |= 1
        self._user_id = value
        self._Modified()
    
    @property
    def project_id(self):
        return self._project_id

    @project_id.setter
    def project_id(self, value):
        self.__field_bitmap0 |= 2
        self._project_id = value
        self._Modified()
    
    @property
    def methods(self):
        return self._methods

    @methods.setter
    def methods(self, value):
        self.__field_bitmap0 |= 4
        if isinstance(value, bytes):
            self._methods = value.decode('utf-8')
        elif isinstance(value, str):
            self._methods = value
        else:
            raise TypeError("%r has type %s, but expected one of: (%s, %s)" % (value, type(value), bytes, str))
        self._Modified()
    
    @property
    def exp(self):
        return self._exp

    @exp.setter
    def exp(self, value):
        self.__field_bitmap0 |= 8
        self._exp = value
        self._Modified()
    

    cdef int _protobuf_deserialize(self, const unsigned char *memory, int size, bint cache):
        cdef int current_offset = 0
        cdef int64_t key
        cdef int64_t field_size
        while current_offset < size:
            key = get_varint64(memory, &current_offset)
            # user_id
            if key == 8:
                self.__field_bitmap0 |= 1
                self._user_id = get_varint32(memory, &current_offset)
            # project_id
            elif key == 16:
                self.__field_bitmap0 |= 2
                self._project_id = get_varint32(memory, &current_offset)
            # methods
            elif key == 26:
                self.__field_bitmap0 |= 4
                field_size = get_varint64(memory, &current_offset)
                self._methods = str(memory[current_offset:current_offset + field_size], 'utf-8')
                current_offset += <int>field_size
            # exp
            elif key == 33:
                self.__field_bitmap0 |= 8
                self._exp = (<double *>&memory[current_offset])[0]
                current_offset += sizeof(double)
            # Unknown field - need to skip proper number of bytes
            else:
                assert skip_generic(memory, &current_offset, size, key & 0x7)

        self._is_present_in_parent = True

        return current_offset

    cpdef void Clear(self):
        """Clears all data that was set in the message."""
        self.reset()
        self._Modified()

    cpdef void ClearField(self, field_name):
        """Clears the contents of a given field."""
        self._clearfield(field_name)
        self._Modified()

    cdef void _clearfield(self, field_name):
        if field_name == 'user_id':
            self._user_id__reset()
        elif field_name == 'project_id':
            self._project_id__reset()
        elif field_name == 'methods':
            self._methods__reset()
        elif field_name == 'exp':
            self._exp__reset()
        else:
            raise ValueError('Protocol message has no "%s" field.' % field_name)

    cpdef void CopyFrom(self, Token other_msg):
        """
        Copies the content of the specified message into the current message.

        Params:
            other_msg (Token): Message to copy into the current one.
        """
        if self is other_msg:
            return
        self.reset()
        self.MergeFrom(other_msg)

    cpdef bint HasField(self, field_name) except -1:
        """
        Checks if a certain field is set for the message.

        Params:
            field_name (str): The name of the field to check.
        """
        if field_name == 'user_id':
            return self.__field_bitmap0 & 1 == 1
        if field_name == 'project_id':
            return self.__field_bitmap0 & 2 == 2
        if field_name == 'methods':
            return self.__field_bitmap0 & 4 == 4
        if field_name == 'exp':
            return self.__field_bitmap0 & 8 == 8
        raise ValueError('Protocol message has no singular "%s" field.' % field_name)

    cpdef bint IsInitialized(self):
        """
        Checks if the message is initialized.

        Returns:
            bool: True if the message is initialized (i.e. all of its required
                fields are set).
        """

    

        return True

    cpdef void MergeFrom(self, Token other_msg):
        """
        Merges the contents of the specified message into the current message.

        Params:
            other_msg: Message to merge into the current message.
        """

        if self is other_msg:
            return

    
        if other_msg.__field_bitmap0 & 1 == 1:
            self._user_id = other_msg._user_id
            self.__field_bitmap0 |= 1
        if other_msg.__field_bitmap0 & 2 == 2:
            self._project_id = other_msg._project_id
            self.__field_bitmap0 |= 2
        if other_msg.__field_bitmap0 & 4 == 4:
            self._methods = other_msg._methods
            self.__field_bitmap0 |= 4
        if other_msg.__field_bitmap0 & 8 == 8:
            self._exp = other_msg._exp
            self.__field_bitmap0 |= 8

        self._Modified()

    cpdef int MergeFromString(self, data, size=None) except -1:
        """
        Merges serialized protocol buffer data into this message.

        Params:
            data (bytes): a string of binary data.
            size (int): optional - the length of the data string

        Returns:
            int: the number of bytes processed during serialization
        """
        cdef int buf
        cdef int length

        length = size if size is not None else len(data)

        buf = self._protobuf_deserialize(data, length, False)

        if buf != length:
            raise DecodeError("Truncated message: got %s expected %s" % (buf, size))

        self._Modified()

        return buf

    cpdef int ParseFromString(self, data, size=None, bint reset=True, bint cache=False) except -1:
        """
        Populate the message class from a string of protobuf encoded binary data.

        Params:
            data (bytes): a string of binary data
            size (int): optional - the length of the data string
            reset (bool): optional - whether to reset to default values before serializing
            cache (bool): optional - whether to cache serialized data

        Returns:
            int: the number of bytes processed during serialization
        """
        cdef int buf
        cdef int length

        length = size if size is not None else len(data)

        if reset:
            self.reset()

        buf = self._protobuf_deserialize(data, length, cache)

        if buf != length:
            raise DecodeError("Truncated message")

        self._Modified()

        if cache:
            self._cached_serialization = data

        return buf

    @classmethod
    def FromString(cls, s):
        message = cls()
        message.MergeFromString(s)
        return message

    cdef void _protobuf_serialize(self, bytearray buf, bint cache):
        # user_id
        if self.__field_bitmap0 & 1 == 1:
            set_varint64(8, buf)
            set_varint32(self._user_id, buf)
        # project_id
        if self.__field_bitmap0 & 2 == 2:
            set_varint64(16, buf)
            set_varint32(self._project_id, buf)
        # methods
        cdef bytes methods_bytes
        if self.__field_bitmap0 & 4 == 4:
            set_varint64(26, buf)
            methods_bytes = self._methods.encode('utf-8')
            set_varint64(len(methods_bytes), buf)
            buf += methods_bytes
        # exp
        if self.__field_bitmap0 & 8 == 8:
            set_varint64(33, buf)
            buf += (<unsigned char *>&self._exp)[:sizeof(double)]

    cpdef void _Modified(self):
        self._is_present_in_parent = True
        self._listener()
        self._cached_serialization = None

    

    cpdef bytes SerializeToString(self, bint cache=False):
        """
        Serialize the message class into a string of protobuf encoded binary data.

        Returns:
            bytes: a byte string of binary data
        """

    

        if self._cached_serialization is not None:
            return self._cached_serialization

        cdef bytearray buf = bytearray()
        self._protobuf_serialize(buf, cache)
        cdef bytes out = bytes(buf)

        if cache:
            self._cached_serialization = out

        return out

    cpdef bytes SerializePartialToString(self):
        """
        Serialize the message class into a string of protobuf encoded binary data.

        Returns:
            bytes: a byte string of binary data
        """
        if self._cached_serialization is not None:
            return self._cached_serialization

        cdef bytearray buf = bytearray()
        self._protobuf_serialize(buf, False)
        return bytes(buf)

    def SetInParent(self):
        """
        Mark this an present in the parent.
        """
        self._Modified()

    def ParseFromJson(self, data, size=None, reset=True):
        """
        Populate the message class from a json string.

        Params:
            data (str): a json string
            size (int): optional - the length of the data string
            reset (bool): optional - whether to reset to default values before serializing
        """
        if size is None:
            size = len(data)
        d = json.loads(data[:size])
        self.ParseFromDict(d, reset)

    def SerializeToJson(self, **kwargs):
        """
        Serialize the message class into a json string.

        Returns:
            str: a json formatted string
        """
        d = self.SerializeToDict()
        return json.dumps(d, **kwargs)

    def SerializePartialToJson(self, **kwargs):
        """
        Serialize the message class into a json string.

        Returns:
            str: a json formatted string
        """
        d = self.SerializePartialToDict()
        return json.dumps(d, **kwargs)

    def ParseFromDict(self, d, reset=True):
        """
        Populate the message class from a Python dictionary.

        Params:
            d (dict): a Python dictionary representing the message
            reset (bool): optional - whether to reset to default values before serializing
        """
        if reset:
            self.reset()

        assert type(d) == dict
        try:
            self.user_id = d["user_id"]
        except KeyError:
            pass
        try:
            self.project_id = d["project_id"]
        except KeyError:
            pass
        try:
            self.methods = d["methods"]
        except KeyError:
            pass
        try:
            self.exp = d["exp"]
        except KeyError:
            pass

        self._Modified()

        return

    def SerializeToDict(self):
        """
        Translate the message into a Python dictionary.

        Returns:
            dict: a Python dictionary representing the message
        """
        out = {}
        if self.__field_bitmap0 & 1 == 1:
            out["user_id"] = self.user_id
        if self.__field_bitmap0 & 2 == 2:
            out["project_id"] = self.project_id
        if self.__field_bitmap0 & 4 == 4:
            out["methods"] = self.methods
        if self.__field_bitmap0 & 8 == 8:
            out["exp"] = self.exp

        return out

    def SerializePartialToDict(self):
        """
        Translate the message into a Python dictionary.

        Returns:
            dict: a Python dictionary representing the message
        """
        out = {}
        if self.__field_bitmap0 & 1 == 1:
            out["user_id"] = self.user_id
        if self.__field_bitmap0 & 2 == 2:
            out["project_id"] = self.project_id
        if self.__field_bitmap0 & 4 == 4:
            out["methods"] = self.methods
        if self.__field_bitmap0 & 8 == 8:
            out["exp"] = self.exp

        return out

    def Items(self):
        """
        Iterator over the field names and values of the message.

        Returns:
            iterator
        """
        yield 'user_id', self.user_id
        yield 'project_id', self.project_id
        yield 'methods', self.methods
        yield 'exp', self.exp

    def Fields(self):
        """
        Iterator over the field names of the message.

        Returns:
            iterator
        """
        yield 'user_id'
        yield 'project_id'
        yield 'methods'
        yield 'exp'

    def Values(self):
        """
        Iterator over the values of the message.

        Returns:
            iterator
        """
        yield self.user_id
        yield self.project_id
        yield self.methods
        yield self.exp

    

    def Setters(self):
        """
        Iterator over functions to set the fields in a message.

        Returns:
            iterator
        """
        def setter(value):
            self.user_id = value
        yield setter
        def setter(value):
            self.project_id = value
        yield setter
        def setter(value):
            self.methods = value
        yield setter
        def setter(value):
            self.exp = value
        yield setter

    



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mmf_serve']

package_data = \
{'': ['*']}

install_requires = \
['PyTurboJPEG>=1.6.5,<2.0.0',
 'XlsxWriter>=3.0.2,<4.0.0',
 'aio-pika>=6.8.1,<7.0.0',
 'aiofile>=3.7.4,<4.0.0',
 'cryptography>=36.0.1,<37.0.0',
 'fastapi>=0.73.0,<0.74.0',
 'mmf-meta>=0.1.3,<0.2.0',
 'opencv-python>=4.5.5,<5.0.0',
 'openpyxl>=3.0.9,<4.0.0',
 'orjson>=3.6.6,<4.0.0',
 'pandas>=1.4.0,<2.0.0',
 'pydantic[dotenv]>=1.9.0,<2.0.0',
 'python-multipart>=0.0.5,<0.0.6',
 'requests>=2.27.1,<3.0.0',
 'xlrd>=2.0.1,<3.0.0']

entry_points = \
{'console_scripts': ['mmfserve = mmf_serve:cli']}

setup_kwargs = {
    'name': 'mmf-serve',
    'version': '0.1.1',
    'description': 'Часть проекта MMF отвечающая за serving',
    'long_description': '# MMF-meta\nЭта библиотека - часть проекта Model Management Framework.\n\nОтвечает за serving\n\n### Пример использования\n```shell\nmmfserve serve_rabbit --n_proc=2 --queue_name=tasks_que --results_exchange=result_exchange main.py\n```\n\n[Подробная документация](https://mm-framework.github.io/docs/)\n',
    'author': 'Викторов Андрей Германович',
    'author_email': 'andvikt@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
